@@ -269,8 +269,8 @@ static void destroy_node(struct fuse *f, fino_t ino, int version)
     struct node *node;
 
     pthread_mutex_lock(&f->lock);
-    node = get_node(f, ino);
-    if(node->version == version && ino != FUSE_ROOT_INO) {
+    node = __get_node(f, ino);
+    if(node && node->version == version && ino != FUSE_ROOT_INO) {
         unhash_name(f, node);
         unhash_ino(f, node);
         free_node(node);
@@ -589,20 +589,27 @@ static void do_getdir(struct fuse *f, struct fuse_in_header *in)
     dh.fuse = f;
     dh.fp = tmpfile();
     dh.dir = in->ino;
-    res = -ENOENT;
-    path = get_path(f, in->ino);
-    if(path != NULL) {
-        res = -ENOSYS;
-        if(f->op.getdir)
-            res = f->op.getdir(path, &dh, (fuse_dirfil_t) fill_dir);
-        free(path);
+    res = -EIO;
+    
+    if (dh.fp == NULL)
+        perror("fuse: failed to create temporary file");
+    else {
+        res = -ENOENT;
+        path = get_path(f, in->ino);
+        if(path != NULL) {
+            res = -ENOSYS;
+            if(f->op.getdir)
+                res = f->op.getdir(path, &dh, (fuse_dirfil_t) fill_dir);
+            free(path);
+        }
+        fflush(dh.fp);
     }
-    fflush(dh.fp);
-
     memset(&arg, 0, sizeof(struct fuse_getdir_out));
-    arg.fd = fileno(dh.fp);
+    if (res == 0)
+        arg.fd = fileno(dh.fp);
     send_reply(f, in, res, &arg, sizeof(arg));
-    fclose(dh.fp);
+    if (dh.fp != NULL)
+        fclose(dh.fp);
 }
 
 static void do_mknod(struct fuse *f, struct fuse_in_header *in,
